require 'rubbis/transaction'
module Rubbis
    class Handler

        attr_reader :client, :buffer, :tx, :server

        def initialize(socket, server)
            @client = socket
            @server = server
            @buffer = ""
            reset_tx!
        end

        def reset_tx!
            @tx = Transaction.new
        end

        def process!(state)
            buffer << client.read_nonblock(1024)
            cmds, processed = Rubbis::Protocol.unmarshal(buffer)

            @buffer = buffer[processed..-1]

            cmds.each do |cmd|
                response = if tx.active?
                    case cmd[0].downcase
                    when 'exec'
                        result = tx.buffer.map do |cmd|
                            dispatch state, cmd
                        end unless tx.dirty?
                        reset_tx!
                        result
                    else
                        tx.queue cmd
                        :queued
                    end
                else
                    dispatch state, cmd
                end

                unless response == :block
                    respond! response
                end

                state.process_list_watches!
            end
        end

        def respond!(response)
            if active?
                server.commit!
                client.write Rubbis::Protocol.marshal(response) 
            end
        end

        def active?
            client
        end

        def disconnect!(state)
            state.unsubscribe_all(self)
            @client = nil
        end

        def dispatch(state, cmd)
            case cmd[0].downcase
            when 'bgsave' then server.bgsave; :ok
            when 'multi' then tx.start!; :ok
            when 'watch' then
                curr_tx = tx
                state.watch(cmd[1]) {
                    tx.dirty! if curr_tx == tx
                }
            else state.apply_command(self, cmd)
            end
        end
    end
end